@@ -4,6 +4,7 @@
 
     @copyright: 2003-2004 by Juergen Hermann <jh@web.de>
                 2009 by ReimarBauer
+                2013 by MoinMoin:ThomasWaldmann
     @license: GNU GPL, see COPYING for details.
 """
 
@@ -19,18 +20,27 @@ class TestEncodePassword(object):
     def testAscii(self):
         """user: encode ascii password"""
         # u'MoinMoin' and 'MoinMoin' should be encoded to same result
-        expected = "{SSHA}xkDIIx1I7A4gC98Vt/+UelIkTDYxMjM0NQ=="
-
-        result = user.encodePassword("MoinMoin", salt='12345')
-        assert result == expected
-        result = user.encodePassword(u"MoinMoin", salt='12345')
-        assert result == expected
+        cfg = self.request.cfg
+        tests = [
+            ('{PASSLIB}', '12345', "{PASSLIB}$6$rounds=1001$12345$jrPUCzPJt1yiixDbzIgSBoKED0/DlNDTHZN3lVarCtN6IM/.LoAw5pgUQH112CErU6wS8HXTZNpqb7wVjHLs/0"),
+            ('{SSHA}', '12345', "{SSHA}xkDIIx1I7A4gC98Vt/+UelIkTDYxMjM0NQ=="),
+        ]
+        for scheme, salt, expected in tests:
+            result = user.encodePassword(cfg, "MoinMoin", salt=salt, scheme=scheme)
+            assert result == expected
+            result = user.encodePassword(cfg, u"MoinMoin", salt=salt, scheme=scheme)
+            assert result == expected
 
     def testUnicode(self):
         """ user: encode unicode password """
-        result = user.encodePassword(u'סיסמה סודית בהחלט', salt='12345') # Hebrew
-        expected = "{SSHA}YiwfeVWdVW9luqyVn8t2JivlzmUxMjM0NQ=="
-        assert result == expected
+        cfg = self.request.cfg
+        tests = [
+            ('{PASSLIB}', '12345', "{PASSLIB}$6$rounds=1001$12345$5srFB66ZCu2JgGwPgdfb1lHRmqkjnKC/RxdsFlWn2WzoQh3btIjH6Ai1LJV9iYLDa9kLP/VQYa4DHLkRnaBw8."),
+            ('{SSHA}', '12345', "{SSHA}YiwfeVWdVW9luqyVn8t2JivlzmUxMjM0NQ=="),
+            ]
+        for scheme, salt, expected in tests:
+            result = user.encodePassword(cfg, u'סיסמה סודית בהחלט', salt=salt, scheme=scheme) # Hebrew
+            assert result == expected
 
 
 class TestLoginWithPassword(object):
@@ -46,6 +56,8 @@ class TestLoginWithPassword(object):
         self.request.user = user.User(self.request)
 
         self.user = None
+        self.passlib_support = self.request.cfg.passlib_support
+        self.password_scheme = self.request.cfg.password_scheme
 
     def teardown_method(self, method):
         """ Run after each test
@@ -97,48 +109,118 @@ class TestLoginWithPassword(object):
     def test_auth_with_apr1_stored_password(self):
         """
         Create user with {APR1} password and check that user can login.
+        Also check if auto-upgrade happens and is saved to disk.
         """
         # Create test user
         name = u'Test User'
+        password = '12345'
         # generated with "htpasswd -nbm blaze 12345"
-        password = '{APR1}$apr1$NG3VoiU5$PSpHT6tV0ZMKkSZ71E3qg.' # 12345
-        self.createUser(name, password, True)
+        pw_hash = '{APR1}$apr1$NG3VoiU5$PSpHT6tV0ZMKkSZ71E3qg.'
+        self.createUser(name, pw_hash, True)
 
         # Try to "login"
-        theuser = user.User(self.request, name=name, password='12345')
+        theuser = user.User(self.request, name=name, password=password)
         assert theuser.valid
+        # Check if the stored password was auto-upgraded on login and saved
+        theuser = user.User(self.request, name=name, password=password)
+        assert theuser.enc_password.startswith(self.password_scheme)
 
     def test_auth_with_md5_stored_password(self):
         """
         Create user with {MD5} password and check that user can login.
+        Also check if auto-upgrade happens and is saved to disk.
         """
         # Create test user
         name = u'Test User'
-        password = '{MD5}$1$salt$etVYf53ma13QCiRbQOuRk/' # 12345
-        self.createUser(name, password, True)
+        password = '12345'
+        pw_hash = '{MD5}$1$salt$etVYf53ma13QCiRbQOuRk/'
+        self.createUser(name, pw_hash, True)
 
         # Try to "login"
-        theuser = user.User(self.request, name=name, password='12345')
+        theuser = user.User(self.request, name=name, password=password)
         assert theuser.valid
+        # Check if the stored password was auto-upgraded on login and saved
+        theuser = user.User(self.request, name=name, password=password)
+        assert theuser.enc_password.startswith(self.password_scheme)
 
     def test_auth_with_des_stored_password(self):
         """
         Create user with {DES} password and check that user can login.
+        Also check if auto-upgrade happens and is saved to disk.
         """
         # Create test user
         name = u'Test User'
+        password = '12345'
         # generated with "htpasswd -nbd blaze 12345"
-        password = '{DES}gArsfn7O5Yqfo' # 12345
-        self.createUser(name, password, True)
+        pw_hash = '{DES}gArsfn7O5Yqfo'
+        self.createUser(name, pw_hash, True)
 
         try:
             import crypt
             # Try to "login"
-            theuser = user.User(self.request, name=name, password='12345')
+            theuser = user.User(self.request, name=name, password=password)
             assert theuser.valid
+            # Check if the stored password was auto-upgraded on login and saved
+            theuser = user.User(self.request, name=name, password=password)
+            assert theuser.enc_password.startswith(self.password_scheme)
         except ImportError:
             py.test.skip("Platform does not provide crypt module!")
 
+    def test_auth_with_sha_stored_password(self):
+        """
+        Create user with {SHA} password and check that user can login.
+        Also check if auto-upgrade happens and is saved to disk.
+        """
+        # Create test user
+        name = u'Test User'
+        password = '12345'
+        pw_hash = '{SHA}jLIjfQZ5yojbZGTqxg2pY0VROWQ='
+        self.createUser(name, pw_hash, True)
+
+        # Try to "login"
+        theuser = user.User(self.request, name=name, password=password)
+        assert theuser.valid
+        # Check if the stored password was auto-upgraded on login and saved
+        theuser = user.User(self.request, name=name, password=password)
+        assert theuser.enc_password.startswith(self.password_scheme)
+
+    def test_auth_with_ssha_stored_password(self):
+        """
+        Create user with {SSHA} password and check that user can login.
+        Also check if auto-upgrade happens and is saved to disk.
+        """
+        # Create test user
+        name = u'Test User'
+        password = '12345'
+        pw_hash = '{SSHA}dbeFtH5EGkOI1jgPADlGZgHWq072TIsKqWfHX7zZbUQa85Ze8774Rg=='
+        self.createUser(name, pw_hash, True)
+
+        # Try to "login"
+        theuser = user.User(self.request, name=name, password=password)
+        assert theuser.valid
+        # Check if the stored password was auto-upgraded on login and saved
+        theuser = user.User(self.request, name=name, password=password)
+        assert theuser.enc_password.startswith(self.password_scheme)
+
+    def test_auth_with_passlib_stored_password(self):
+        """
+        Create user with {PASSLIB} password and check that user can login.
+        """
+        if not self.passlib_support:
+            py.test.skip("test requires passlib, but passlib_support is False")
+        # Create test user
+        name = u'Test User'
+        password = '12345'
+        pw_hash = '{PASSLIB}$6$rounds=1001$/AVWSh/RUWpcppfl$8DCRGLaBD3KoV4Ag67sUv6b2QdrUFXk1yWCxqWnBLJ.iHSe4Piv6nqzSQgELeLPIvwTC9APaWv1XCTOHjkLOj/'
+        self.createUser(name, pw_hash, True)
+
+        # Try to "login"
+        theuser = user.User(self.request, name=name, password=password)
+        assert theuser.valid
+        # Check if the stored password was auto-upgraded on login and saved
+        theuser = user.User(self.request, name=name, password=password)
+        assert theuser.enc_password.startswith(self.password_scheme)
+
     def testSubscriptionSubscribedPage(self):
         """ user: tests isSubscribedTo  """
         pagename = u'HelpMiscellaneous'
@@ -179,63 +261,6 @@ class TestLoginWithPassword(object):
 
         assert not theUser.exists()
 
-    def test_upgrade_password_from_sha_to_ssha(self):
-        """
-        Create user with {SHA} password and check that logging in
-        upgrades to {SSHA}.
-        """
-        name = u'/no such user/'
-        password = '{SHA}jLIjfQZ5yojbZGTqxg2pY0VROWQ=' # 12345
-        self.createUser(name, password, True)
-
-        # User is not required to be valid
-        theuser = user.User(self.request, name=name, password='12345')
-        assert theuser.enc_password[:6] == '{SSHA}'
-
-    def test_upgrade_password_from_apr1_to_ssha(self):
-        """
-        Create user with {APR1} password and check that logging in
-        upgrades to {SSHA}.
-        """
-        # Create test user
-        name = u'Test User'
-        # generated with "htpasswd -nbm blaze 12345"
-        password = '{APR1}$apr1$NG3VoiU5$PSpHT6tV0ZMKkSZ71E3qg.' # 12345
-        self.createUser(name, password, True)
-
-        # User is not required to be valid
-        theuser = user.User(self.request, name=name, password='12345')
-        assert theuser.enc_password[:6] == '{SSHA}'
-
-    def test_upgrade_password_from_md5_to_ssha(self):
-        """
-        Create user with {MD5} password and check that logging in
-        upgrades to {SSHA}.
-        """
-        # Create test user
-        name = u'Test User'
-        password = '{MD5}$1$salt$etVYf53ma13QCiRbQOuRk/' # 12345
-        self.createUser(name, password, True)
-
-        # User is not required to be valid
-        theuser = user.User(self.request, name=name, password='12345')
-        assert theuser.enc_password[:6] == '{SSHA}'
-
-    def test_upgrade_password_from_des_to_ssha(self):
-        """
-        Create user with {DES} password and check that logging in
-        upgrades to {SSHA}.
-        """
-        # Create test user
-        name = u'Test User'
-        # generated with "htpasswd -nbd blaze 12345"
-        password = '{DES}gArsfn7O5Yqfo' # 12345
-        self.createUser(name, password, True)
-
-        # User is not required to be valid
-        theuser = user.User(self.request, name=name, password='12345')
-        assert theuser.enc_password[:6] == '{SSHA}'
-
     def test_for_email_attribute_by_name(self):
         """
         checks for no access to the email attribute by getting the user object from name
@@ -269,7 +294,7 @@ class TestLoginWithPassword(object):
         self.user.name = name
         self.user.email = email
         if not pwencoded:
-            password = user.encodePassword(password)
+            password = user.encodePassword(self.request.cfg, password)
         self.user.enc_password = password
 
         # Validate that we are not modifying existing user data file!
