@@ -7,7 +7,9 @@
     @copyright: 2005-2007 MoinMoin:ThomasWaldmann
     @license: GNU GPL, see COPYING for details.
 """
-import logging
+
+from MoinMoin import log
+logging = log.getLogger(__name__)
 
 import os, codecs, errno
 from MoinMoin import config, wikiutil
@@ -75,7 +77,7 @@ class LineBuffer:
         self.offsets = offsets
         self.len = linecount
         # Decode lines after offset in file is calculated
-        self.lines = [unicode(line, config.charset) for line in lines]
+        self.lines = [unicode(line.rstrip('\n'), config.charset) for line in lines]
 
 
 class LogFile:
@@ -157,7 +159,7 @@ class LogFile:
                     raise
             return self._input
         elif name == "_output":
-            self._output = codecs.open(self.__filename, 'a', config.charset)
+            self._output = codecs.open(self.__filename, 'ab', config.charset)
             return self._output
         else:
             raise AttributeError(name)
@@ -165,7 +167,7 @@ class LogFile:
     def size(self):
         """ Return log size in bytes
 
-        Return 0 if the file does not exists. Raises other OSError.
+        Return 0 if the file does not exist. Raises other OSError.
 
         @return: size of log file in bytes
         @rtype: Int
@@ -180,7 +182,7 @@ class LogFile:
     def lines(self):
         """ Return number of lines in the log file
 
-        Return 0 if the file does not exists. Raises other OSError.
+        Return 0 if the file does not exist. Raises other OSError.
 
         Expensive for big log files - O(n)
 
@@ -202,6 +204,7 @@ class LogFile:
             raise
 
     def date(self):
+        # ToDo check if we need this method
         """ Return timestamp of log file in usecs """
         try:
             mtime = os.path.getmtime(self.__filename)
