@@ -174,7 +174,7 @@ class EditLog(LogFile):
         """
         if request.cfg.log_remote_addr or self.force_ip:
             if host is None:
-                host = request.remote_addr
+                host = request.remote_addr or ''
 
             if request.cfg.log_reverse_dns_lookups:
                 import socket
