@@ -37,15 +37,16 @@ class EventLog(LogFile):
 
         if values is None:
             values = {}
-        if add_http_info:
-            # All these are ascii
+        if request.cfg.log_remote_addr and add_http_info:
+            # if cfg.log_remote_addr is False (usually for privacy reasons),
+            # we likely do not want to log user agent and http referer either.
             for key in ['remote_addr', 'http_user_agent', 'http_referer']:
                 value = getattr(request, key, '')
                 if value:
                     # Save those http headers in UPPERcase
                     values[key.upper()] = value
         # Encode values in a query string TODO: use more readable format
-        values = wikiutil.makeQueryString(values, want_unicode=True)
+        values = wikiutil.makeQueryString(values)
         self._add(u"%d\t%s\t%s\n" % (mtime_usecs, eventtype, values))
 
     def parser(self, line):
