@@ -6,12 +6,12 @@
     according to the edit log; if you shortened the log, the displayed
     information may not be what you expect.
 
-    @copyright: 2001 by Jrgen Hermann <jh@web.de>
+    @copyright: 2001 Juergen Hermann <jh@web.de>
     @license: GNU GPL, see COPYING for details.
 """
 
 from MoinMoin.macro import RecentChanges
 
-def execute(macro, args):
-    return RecentChanges.execute(macro, args, abandoned=1)
+def macro_AbandonedPages(macro):
+    return RecentChanges.macro_RecentChanges(macro, abandoned=True)
 
