@@ -13,7 +13,7 @@ from MoinMoin.action.AttachFile import do_admin_browser
 
 Dependencies = ["time"]
 
-def execute(macro, args):
+def macro_SystemAdmin(macro):
     _ = macro.request.getText
     request = macro.request
 
@@ -22,10 +22,10 @@ def execute(macro, args):
         return ''
 
     _MENU = {
-        'attachments': (("File attachment browser"), do_admin_browser),
-        'users': (("User account browser"), do_user_browser),
+        'attachments': (_("File attachment browser"), do_admin_browser),
+        'users': (_("User account browser"), do_user_browser),
     }
-    choice = request.form.get('sysadm', [None])[0]
+    choice = request.values.get('sysadm')
 
     # create menu
     menuitems = [(label, fnid) for fnid, (label, handler) in _MENU.items()]
