@@ -2,11 +2,11 @@
 """
     Outputs the text verbatimly.
 
-    @copyright: 2005 by Thomas Waldmann
+    @copyright: 2005 MoinMoin:ThomasWaldmann
     @license: GNU GPL, see COPYING for details
 """
 
 Dependencies = []
 
-def execute(macro, args):
-    return macro.formatter.escapedText(args)
+def macro_Verbatim(macro, text=u''):
+    return macro.formatter.escapedText(text)
