@@ -27,9 +27,28 @@ class Parser:
         self.request = request
         self.form = request.form
         self._ = request.getText
+        self.start_line = kw.get('start_line', 0)
 
-    def format(self, formatter):
+    def format(self, formatter, **kw):
         """ Send the text. """
+
+        self.lines = self.raw.expandtabs().split('\n')
+        if self.lines[-1] == '':
+            del self.lines[-1]
+
+        self.lineno = self.start_line
+
         self.request.write(formatter.preformatted(1))
-        self.request.write(formatter.text(self.raw.expandtabs()))
+
+        for lineno in range(1, self.start_line + 1):
+            self.request.write(formatter.line_anchordef(lineno))
+
+        for line in self.lines:
+            if self.lineno != self.start_line:
+                self.request.write(formatter.text('\n'))
+
+            self.lineno += 1
+            self.request.write(formatter.line_anchordef(self.lineno))
+            self.request.write(formatter.text(line))
+
         self.request.write(formatter.preformatted(0))
