@@ -22,7 +22,7 @@ Detailed Instructions:
 General syntax: moin [options] maint cleanpage [cleanpage-options]
 
 [options] usually should be:
-    --config-dir=/path/to/my/cfg/ --wiki-url=wiki.example.org/
+    --config-dir=/path/to/my/cfg/ --wiki-url=http://wiki.example.org/
 
 [cleanpage-options] see below:
     0. Verify the outputted shell script before running it.
