@@ -217,7 +217,7 @@ class Parser:
         inline = url[0] == 'i'
         drawing = url[0] == 'd'
         url = url.split(":", 1)[1]
-        url = wikiutil.url_unquote(url, want_unicode=True)
+        url = wikiutil.url_unquote(url)
         text = text or url
 
         from MoinMoin.action import AttachFile
@@ -748,7 +748,7 @@ class Parser:
 
     def _heading_repl(self, word):
         """Handle section headings."""
-        import sha
+        from MoinMoin.support.python_compatibility import hash_new
 
         h = word.strip()
         level = 1
@@ -768,7 +768,7 @@ class Parser:
         if self.titles[pntt] > 1:
             unique_id = '-%d' % self.titles[pntt]
         result = self._closeP()
-        result += self.formatter.heading(1, depth, id="head-"+sha.new(pntt.encode(config.charset)).hexdigest()+unique_id)
+        result += self.formatter.heading(1, depth, id="head-"+hash_new('sha1', pntt.encode(config.charset)).hexdigest()+unique_id)
 
         return (result + self.formatter.text(title_text) +
                 self.formatter.heading(0, depth))
