@@ -9,6 +9,7 @@
 
 import sys, os, shutil, time, errno
 from stat import S_ISDIR, ST_MODE, S_IMODE
+import warnings
 
 from MoinMoin import log
 logging = log.getLogger(__name__)
@@ -29,131 +30,7 @@ def chmod(name, mode, catchexception=True):
             raise
 
 
-# begin copy of werkzeug.posixemulation from werkzeug 0.6.1(pre) repo
-r"""
-    werkzeug.posixemulation
-    ~~~~~~~~~~~~~~~~~~~~~~~
-
-    Provides a POSIX emulation for some features that are relevant to
-    web applications.  The main purpose is to simplify support for
-    systems such as Windows NT that are not 100% POSIX compatible.
-
-    Currently this only implements a :func:`rename` function that
-    follows POSIX semantics.  Eg: if the target file already exists it
-    will be replaced without asking.
-
-    This module was introduced in 0.6.1 and is not a public interface.
-    It might become one in later versions of Werkzeug.
-
-    :copyright: (c) 2010 by the Werkzeug Team, see AUTHORS for more details.
-    :license: BSD, see LICENSE for more details.
-"""
-import os
-import errno
-import random
-
-
-can_rename_open_file = False
-if os.name == 'nt':
-    _rename = lambda src, dst: False
-    _rename_atomic = lambda src, dst: False
-
-    try:
-        import ctypes
-        _GetLastError = ctypes.windll.kernel32.GetLastError
-
-        def _LogLastError(fn):
-            err = _GetLastError()
-            logging.debug("%s returned: %r" % (fn, err))
-            # 5 = Access Denied
-            # 6800 = The function attempted to use a name that is reserved
-            #        for use by another transaction.
-
-        _MOVEFILE_REPLACE_EXISTING = 0x1
-        _MOVEFILE_WRITE_THROUGH = 0x8
-        _MoveFileEx = ctypes.windll.kernel32.MoveFileExW
-
-        def _rename(src, dst):
-            if not isinstance(src, unicode):
-                src = unicode(src, sys.getfilesystemencoding())
-            if not isinstance(dst, unicode):
-                dst = unicode(dst, sys.getfilesystemencoding())
-            if _rename_atomic(src, dst):
-                return True
-            logging.debug("PSEUDO-atomic rename %r %r" % (src, dst))
-            retry = 0
-            ret = 0
-            while not ret and retry < 100:
-                ret = _MoveFileEx(src, dst, _MOVEFILE_REPLACE_EXISTING |
-                                            _MOVEFILE_WRITE_THROUGH)
-                if ret == 0:
-                    _LogLastError("MoveFileExW")
-                    time.sleep(0.001)
-                    retry += 1
-                    logging.debug("retry %d after sleeping" % retry)
-            return ret
-
-        # this stuff only exists in windows >= vista / windows server 2008
-        _CreateTransaction = ctypes.windll.ktmw32.CreateTransaction
-        _CommitTransaction = ctypes.windll.ktmw32.CommitTransaction
-        _MoveFileTransacted = ctypes.windll.kernel32.MoveFileTransactedW
-        _CloseHandle = ctypes.windll.kernel32.CloseHandle
-        can_rename_open_file = True
-
-        def _rename_atomic(src, dst):
-            ta = _CreateTransaction(None, 0, 0, 0, 0, 1000, 'Werkzeug rename')
-            if ta == -1:
-                _LogLastError("CreateTransaction")
-                return False
-            try:
-                logging.debug("atomic rename %r %r" % (src, dst))
-                retry = 0
-                ret = 0
-                while not ret and retry < 100:
-                    ret = _MoveFileTransacted(src, dst, None, None,
-                                               _MOVEFILE_REPLACE_EXISTING |
-                                               _MOVEFILE_WRITE_THROUGH, ta)
-                    if ret == 0:
-                        _LogLastError("MoveFileTransacted")
-                        time.sleep(0.001)
-                        retry += 1
-                        logging.debug("retry %d after sleeping" % retry)
-                    else:
-                        ret = _CommitTransaction(ta)
-                        if ret == 0:
-                            _LogLastError("CommitTransaction")
-                return ret
-            finally:
-                ret = _CloseHandle(ta)
-                if ret == 0:
-                    _LogLastError("CloseHandle")
-    except Exception:
-        pass
-
-    def rename(src, dst):
-        # Try atomic or pseudo-atomic rename
-        if _rename(src, dst):
-            return
-        # Fall back to "move away and replace"
-        logging.debug("NON-atomic rename %r %r" % (src, dst))
-        try:
-            os.rename(src, dst)
-        except OSError, e:
-            if e.errno != errno.EEXIST:
-                raise
-            old = "%s-%08x" % (dst, random.randint(0, sys.maxint))
-            os.rename(dst, old)
-            os.rename(src, dst)
-            try:
-                os.unlink(old)
-            except Exception:
-                pass
-else:
-    #logging.debug("atomic os.rename (POSIX)")
-    rename = os.rename
-    can_rename_open_file = True
-
-# end copy of werkzeug.posixemulation
+from werkzeug.posixemulation import *
 
 rename_overwrite = rename
 
@@ -357,7 +234,7 @@ def copytree(src, dst, symlinks=False):
         except (IOError, os.error), why:
             errors.append((srcname, dstname, why))
     if errors:
-        raise EnvironmentError, errors
+        raise EnvironmentError(str(errors))
 
 # Code could come from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/65203
 
@@ -402,21 +279,29 @@ else:
     def realPathCase(path):
         return None
 
-# dircache stuff seems to be broken on win32 (at least for FAT32, maybe NTFS)
-DCENABLED = 1 # set to 0 to disable dirchache usage
+# dircache stuff seems to be broken on win32 (at least for FAT32, maybe NTFS).
+# dircache stuff is also broken on POSIX if updates happen too fast (< 1s).
+DCENABLED = 0 # set to 0 to completely disable dircache usage
+
+# Note: usage of the dc* functions below is deprecated, they'll get removed soon.
+dc_deprecated = "dircache function calls (dcdisable,dclistdir,dcreset) are deprecated, please fix caller"
+
 def dcdisable():
+    warnings.warn(dc_deprecated, DeprecationWarning, stacklevel=2)
     global DCENABLED
     DCENABLED = 0
 
 import dircache
 
 def dclistdir(path):
+    warnings.warn(dc_deprecated, DeprecationWarning, stacklevel=2)
     if sys.platform == 'win32' or not DCENABLED:
         return os.listdir(path)
     else:
         return dircache.listdir(path)
 
 def dcreset():
+    warnings.warn(dc_deprecated, DeprecationWarning, stacklevel=2)
     if sys.platform == 'win32' or not DCENABLED:
         return
     else:
